<?php
# @Author: consentio
# @Date:   2018-08-18T16:58:32+02:00
# @Last modified by:   consentio
# @Last modified time: 2024-01-29T17:43:43+01:00

/**
 * plugin import_csv
 * @package consentio
 */

class import_csv {
	private $file = '';
	private $delimiter = '';
	private $createID = "`id` bigint(20) unsigned NOT NULL AUTO_INCREMENT";
	private $createField = "varchar(255) DEFAULT NULL";
	private $createPrimary = "PRIMARY KEY (`id`)";
	private $createEng = "ENGINE=MyISAM DEFAULT CHARSET=utf8";

	public function __construct(){

		}

	public function plugin($html) {
		$todo = clean_var(param('todo'), 'w');
		if (in_array($todo, ['nt', 'imp', 'comp', 'alter'])) {$html['action'] = 'pro';}
		if ($todo == 'deps') {$html['action'] = 'deps';}
		switch ($html['action']) {
			case "pro": $html = $this->process_dialogue($html); break;
			case "deps": $html = $this->more_depending_inputs($html); break;
			default: $html = $this->import_dialogue($html);
			}
		return $html;
		}

	private function import_dialogue($html) {
		$todo = 'nt';
		$file = clean_var(param('csv_file'), 'w');
		$delimiter = param('delimiter') ? param('delimiter') : ';';
		$feedback = '';
		if (array_key_exists("filecsv_file", $_FILES) and $_FILES["filecsv_file"]['size']) {
			$file = upload($html['dir'],"filecsv_file", 'u');
			}
		if (!file_exists($html['dir']."/$file")) {$feedback = "Missing CSV File ".$html['dir']."/$file";}
		if (!$file) {$feedback = "Upload a CSV-File";}
		else {$this->file = $file; $this->delimiter = $delimiter;}

		if ($feedback) {
			$todo = 'up';
			$form = [
				"title"			=> "Import CSV",
				"html"			=> $html,
				"feedback"		=> $feedback,
				"button_text"	=> i18n('next'),
				"no_cancel"		=> 1	
				];

			$inputs = [
				"csv_file"	=> [
					"caption" => i18n('file'),
					"type" => "file",
					"duty"			=> 1
					],
				"delimiter" => [
					'type'	=> 'array',
					'extra'	=> [';',',','\t'],
					'raw'	=> true,
					'default' => $delimiter
					]
				];

			$obj = new access($form);
			$obj->add_input($inputs);
			$html = $obj->form();
			}

		else {$html = $this->show_csv();}
		$html['left'] = $this->tree($html, $todo);
		return $html;
		}

	private function process_dialogue($html) {
		$todo = clean_var(param('todo'), 'w');
		if (!$todo) {$todo = 'nt';}
		$file = clean_var(param('csv_file'), 'w');
		$table = clean_var(param('table'), 'w');
		$delimiter = param('delimiter') ? param('delimiter') : ';';
		$feedback = '';
		$this->file = $file; $this->delimiter = $delimiter;

		if (!$table) {$feedback = "Enter a Table Name";}

		if ($feedback) {
			$tables = $this->get_tables();
			$form = [
				"title"			=> "Import CSV",
				"html"			=> $html,
				"feedback"		=> $feedback,
				"button_text"	=> i18n('next'),
				"no_cancel"		=> 1
				];

				$inputs = [
					"csv_file"	=> [
						"type" => "hidden",
						"duty"			=> 1,
						"default"	=> $file
						],
					"table"	=> [
						'type'	=> 'array',
						'extra'	=> $tables,
						'raw'	=> 1,
						"duty"			=> 1,
						"default"	=> $table
						],
					"delimiter" => [
						'type'	=> 'hidden',
						'default' => $delimiter
						],
					"truncate"=> [
						"caption" => 'TRUNCATE',
						"type" => "flag"
					],
					"todo"=> [
						"type" => "free",
						'default' => $todo
						]
					];

			if ($todo == 'nt') {
				$inputs['table']['type'] = 'text';
				unset($inputs['truncate']);
				}
			if ($todo == 'alter' || $todo == 'comp') {
				$csv_cols = $this->get_csv_cols();
				unset($inputs['truncate']);
				$inputs['table']['css_class'] = 'ask_for_more';
				$inputs['csv_id'] = [
					"caption" => 'ID of CSV table',
					'type'	=> 'array',
					'extra'	=> $csv_cols,
					'raw'	=> 1,
					"duty"			=> 1
					];
				$inputs['csv_val'] = [
					"caption" => 'CSV value',
					'type'	=> 'array',
					'extra'	=> $csv_cols,
					'raw'	=> 1,
					"duty"			=> 1
					];
				}
			$obj = new access($form);
			$obj->add_input($inputs);
			$html = $obj->form();
			}
		else {
			if ($todo == 'alter') {$html = $this->update_csv($table);}
			elseif ($todo == 'comp') {
				$html = $this->compare_csv($table);
				$execute = '';
				foreach ($_POST as $k=>$v) {
						if (!in_array($k, ['rid', '_store'])) {
							if ($k == 'todo') {$v = 'alter';}
							$execute .= "<input type='hidden' name='$k' value='$v' />";
							}
						// rid: 0_id: import_csvsid: 202401291709370001csv_file: Lager230129.csvdelimiter: ;todo: comptable: productstable_id: product_notable_val: stockcsv_id: Artikelnrcsv_val: Lager_store: 1
						}
				$execute .= "<button type='submit' name='_store' value='1'><i class='fas fa-cog fa-lg'></i>".i18n('execute', true)."</button>&nbsp;";
				$html['content'] .= htmltag('form', "action='index.php' method='post' enctype='multipart/form-data'", $execute);
				}
			else {$html = $this->read_csv($table);}
			}
		$html['left'] = $this->tree($html, $todo);
		if ($todo == 'alter' || $todo == 'comp') {
			$html['scripts'] .= <<< "EOL"
			<script type="text/javascript">
			\$(document).ready(function(){\$(".ask_for_more").moreInputs({'id':"import_csv",'sid':"{$html['sid']}",'script':"{$html['myurl']}",'todo':'deps'})});
			</script>
			EOL;
			}
		return $html;
		}

	private function show_csv() {
		global $html;
		$CSV = fopen($html['dir']."/$this->file", 'r');
		$i = 0;
		$table = "";
		$f = 0;
		while (($data = fgetcsv($CSV, 1000, $this->delimiter)) !== FALSE) {
			$cell = $i ? 'td' : 'th';
			$tr = '';
			foreach ($data as $field) {$tr .= htmltag($cell, '', $field);}
			$table .= htmltag('tr', '', $tr);
			$i++;
			}
		fclose($CSV);
		$html['formtitle'] = 'Import CSV';
		$html['content'] = htmltag('p', 'class="green"', "$i rows");
		$html['content'] .= htmltag('table', 'class="table"', $table);
		return $html;
		}

	private function read_csv($table) {
		global $html, $db;
		$CSV = fopen($html['dir']."/$this->file", 'r');
		$i = 0;
		$c = "";
		$f = 0;
		$fields = [];
		$exfields = '';
		if (param('truncate') == 'on') {
			$q = "TRUNCATE TABLE `$table`;";
			$arg = $db->prepare($q);
			$check = $arg->execute();
			if ($check === false) {
				$err = $arg->errorInfo();
				$error = array_key_exists(2, $err) ? $err[2] : "";
				$c .= "<p class='red'>".i18n('error').": $error<br/>$q</p>";
				}
			}

		while (($data = fgetcsv($CSV, 1000, $this->delimiter)) !== FALSE) {
			if (!$i) {
				$j = 1;
				foreach ($data as $field) {
					$field = ansitrans($field);
					if (!$field) {$field = "unknown$j"; $j++;}
					array_push($fields, $field);
					}
				$exfields = implode(', ', array_map(function($val){return sprintf("`%s`", $val);}, $fields));
				if (param('todo') == 'nt') {
					$q = "$this->createID, ";
					foreach ($fields as $k) {$q .= " `$k` ".$this->createField.", ";}
					$q .= $this->createPrimary;
					$q = "CREATE TABLE IF NOT EXISTS `$table` ($q) ". $this->createEng.";";
					$check = true;
					$arg = $db->prepare($q);
					try {$arg->execute();}
					catch (PDOException $e) {$check = false;}
					if ($check === false) {
						$err = $arg->errorInfo();
						$error = array_key_exists(2, $err) ? $err[2] : "";
						$c .= "<p class='red'>".i18n('error').": $error<br/>$q</p>";
						$f = 1;
						}
					else {$c .= "<p class='green'>CREATE TABLE `$table`: OK</p>";}
					}
				}
			else {
				$check = true;
				$values = '';
				$row = [];
				$i = 0;
				foreach ($data as $field) {
					$field = rtrim($field);
					if (!$field) {$field = NULL;}
					array_push($row, $field);
					$values .= "?, ";
					}

				$values = preg_replace("/\, $/", "", $values);
				$insert = "INSERT INTO `$table` ($exfields) VALUES ($values);";
				$arg = $db->prepare($insert);
				try {$arg->execute($row);}
				catch (PDOException $e) {$check = false;}
				if ($check === false) {
					$err = $arg->errorInfo();
					$error = array_key_exists(2, $err) ? $err[2] : "";
					$c .= "<p class='red'>Record #$i: ".i18n('error').": $error $data</p>";
					}
				}
			$i++;
			}
		fclose($CSV);
		$html['formtitle'] = 'Import CSV';
		$html['content'] = $c.htmltag('p', 'class="green"', "$i rows");
		$html['left'] = $this->tree($html, '');
		return $html;
		}

	private function update_csv($table) {
		global $html, $db;
		$tableID = clean_var(param('table_id'), 'w');
		$tableVAL = clean_var(param('table_val'), 'w');
		$csvID = ansitrans(clean_var(param('csv_id'), 'w'));
		$csvVAL = ansitrans(clean_var(param('csv_val'), 'w'));
		$CSV = fopen($html['dir']."/$this->file", 'r');
		$i = 0;
		$c = "";
		$fields = [];
		while (($data = fgetcsv($CSV, 1000, $this->delimiter)) !== FALSE) {
			if (!$i) {
				$j = 1;
				foreach ($data as $field) {
					$field = ansitrans($field);
					if (!$field) {$field = "unknown$j"; $j++;}
					array_push($fields, $field);
					}
				}
			else {
				$check = true;
				$row = [];
				$j = 0;
				foreach ($data as $field) {
					$field = rtrim($field);
					if (!$field) {$field = NULL;}
					$row[$fields[$j]] = $field;
					$j++;
					}
				$update = "UPDATE `$table` SET `$tableVAL`  = '$row[$csvVAL]' WHERE `$tableID` = '$row[$csvID]';";
				$arg = $db->prepare($update);
				try {$arg->execute();}
				catch (PDOException $e) {$check = false;}
				if ($check === false) {
					$err = $arg->errorInfo();
					$error = array_key_exists(2, $err) ? $err[2] : "";
					$c .= "<p class='red'>Record #$i: ".i18n('error').": $error $update</p>";
					}
				}
			$i++;
			}
		fclose($CSV);
		$html['formtitle'] = 'Import CSV';
		$html['content'] = $c.htmltag('p', 'class="green"', "$i rows");
		$html['left'] = $this->tree($html, '');
		return $html;
		}

	private function compare_csv($table) {
		global $html, $db;
		$tableID = clean_var(param('table_id'), 'w');
		$tableVAL = clean_var(param('table_val'), 'w');
		$csvID = ansitrans(clean_var(param('csv_id'), 'w'));
		$csvVAL = ansitrans(clean_var(param('csv_val'), 'w'));
		$CSV = fopen($html['dir']."/$this->file", 'r');
		$i = 0;
		$c = "";
		$fields = [];
		while (($data = fgetcsv($CSV, 1000, $this->delimiter)) !== FALSE) {
			if (!$i) {
				$j = 1;
				foreach ($data as $field) {
					$field = ansitrans($field);
					if (!$field) {$field = "unknown$j"; $j++;}
					array_push($fields, $field);
					}
					$c .= htmltag('tr', '', "<th>$tableID</th><th>$tableVAL</th><th>$csvVAL</th>");
				}
			else {
				$check = true;
				$row = [];
				$j = 0;
				foreach ($data as $field) {
					$field = rtrim($field);
					$row[$fields[$j]] = $field;
					$j++;
					}
				$q = "SELECT `$tableID`, `$tableVAL` FROM `$table` WHERE `$tableID` = '$row[$csvID]';";
				$arg = $db->prepare($q);
				try {$arg->execute();}
				catch (PDOException $e) {$check = false;}
				if ($check === false) {
					$err = $arg->errorInfo();
					$error = array_key_exists(2, $err) ? $err[2] : "";
					$c .= "<p class='red'>Record #$i: ".i18n('error').": $error $update</p>";
					}
				else {
					$data = $arg->fetch(PDO::FETCH_ASSOC);
					$col = $row[$csvVAL] != $data[$tableVAL] ? 'red' : 'green';
					$c .= htmltag('tr', '', "<td>$data[$tableID]</td><td>$data[$tableVAL]</td><td class=\"$col\">$row[$csvVAL]</td>");
					}
				}
			$i++;
			}
		fclose($CSV);
		$html['formtitle'] = 'Compare CSV';
		$html['content'] = htmltag('table', 'class="table"', $c);
		$html['left'] = $this->tree($html, '');
		return $html;
		}

	private function tree($html, $todo) {
		$state = 'closed';
		if ($todo == 'up') {
			$state = 'open';
			$list = [];
			}
		else {$list = ['New Table' => 'nt', 'Import into Table' => 'imp', 'Compare Values of a Table' => 'comp', 'Alter Values of a Table' => 'alter'];}
		$item = [
				"caption"	=> 'Start',
				"link"		=> "{$html['myurl']}?_id=import_csv&amp;sid={$html['sid']}",
				"state"		=> $state
				];
		$tree = list_item($item);
		foreach ($list as $k => $v) {
			$state = $todo == $v ? 'open' : 'closed';
			$item = [
				"caption"	=> $k,
				"link"		=> "{$html['myurl']}?_id=import_csv&amp;sid={$html['sid']}&amp;todo=$v&amp;csv_file=$this->file&amp;delimiter=$this->delimiter",
				"state"		=> $state
				];
			$tree .= list_item($item);
			}
		if ($tree) {$tree = parse_in('gui_tree', ['treelist' => $tree]);}
		$tree .= say_mess('You may ruin all data of a table!', 'Proceed with Caution', 'bomb-red');
		return $tree;
		}

	private function get_tables() {
		global $db;
		$arg = $db->prepare("SHOW TABLES;");
		$arg->execute();
		$tables = $arg->fetchAll(PDO::FETCH_COLUMN, 0);
		return $tables;
		}

	private function more_depending_inputs($html) {
		$val = param('_v');
		$prae = param('_ref');
		$cols = $this->get_fields($val);

		$form = [
			"html"	=> $html
			];
		$inputs = [
			$prae.'_id' =>	[
				'caption' => 'ID of table',
				'type'	=> 'array',
				'extra'	=> $cols,
				'raw'	=> 1,
				'duty'	=> 1
				],
			$prae.'_val' =>	[
				'caption' => 'Field to be altered in table',
				'type'	=> 'array',
				'extra'	=> $cols,
				'raw'	=> 1,
				'duty'	=> 1
				],
			];
		$obj = new access($form);
		$obj->add_input($inputs);
		$html['content'] = $obj->inputs;
		$html['gui'] = 6;
		return $html;
		}

	private function get_csv_cols() {
		global $html;
		$CSV = fopen($html['dir']."/$this->file", 'r');
		$cols = fgetcsv($CSV, 1000, $this->delimiter);
		fclose($CSV);
		return $cols;
		}

	private function get_fields($table) {
		global $db;
		$q = "SHOW FULL COLUMNS FROM `$table`;";
		$arg = $db->prepare($q);
		$arg->execute();
		$cols = $arg->fetchAll(PDO::FETCH_COLUMN, 0);
		return $cols;
		}
	}
?>
